package com.sun.jmx.remote.util;

import java.io.OutputStream;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.SortedMap;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Iterator;
import com.sun.jmx.remote.security.NotificationAccessController;
import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import javax.management.InstanceNotFoundException;
import javax.management.ObjectName;
import javax.management.MBeanServer;
import java.util.Map;
import java.util.SortedSet;

public class EnvHelp
{
    public static final String CREDENTIAL_TYPES = "jmx.remote.rmi.server.credential.types";
    private static final String DEFAULT_CLASS_LOADER = "jmx.remote.default.class.loader";
    private static final String DEFAULT_CLASS_LOADER_NAME = "jmx.remote.default.class.loader.name";
    public static final String BUFFER_SIZE_PROPERTY = "jmx.remote.x.notification.buffer.size";
    public static final String MAX_FETCH_NOTIFS = "jmx.remote.x.notification.fetch.max";
    public static final String FETCH_TIMEOUT = "jmx.remote.x.notification.fetch.timeout";
    public static final String NOTIF_ACCESS_CONTROLLER = "com.sun.jmx.remote.notification.access.controller";
    public static final String DEFAULT_ORB = "java.naming.corba.orb";
    public static final String HIDDEN_ATTRIBUTES = "jmx.remote.x.hidden.attributes";
    public static final String DEFAULT_HIDDEN_ATTRIBUTES = "java.naming.security.* jmx.remote.authenticator jmx.remote.context jmx.remote.default.class.loader jmx.remote.message.connection.server jmx.remote.object.wrapping jmx.remote.rmi.client.socket.factory jmx.remote.rmi.server.socket.factory jmx.remote.sasl.callback.handler jmx.remote.tls.socket.factory jmx.remote.x.access.file jmx.remote.x.password.file ";
    private static final SortedSet<String> defaultHiddenStrings;
    private static final SortedSet<String> defaultHiddenPrefixes;
    public static final String SERVER_CONNECTION_TIMEOUT = "jmx.remote.x.server.connection.timeout";
    public static final String CLIENT_CONNECTION_CHECK_PERIOD = "jmx.remote.x.client.connection.check.period";
    public static final String JMX_SERVER_DAEMON = "jmx.remote.x.daemon";
    private static final ClassLogger logger;
    
    public EnvHelp() {
    }
    
    public static ClassLoader resolveServerClassLoader(Map<String, ?> map, MBeanServer mBeanServer) throws InstanceNotFoundException {
        if (map == null) {
            return Thread.currentThread().getContextClassLoader();
        }
        Object value = map.get("jmx.remote.default.class.loader");
        Object value2 = map.get("jmx.remote.default.class.loader.name");
        if (value != null && value2 != null) {
            throw new IllegalArgumentException("Only one of jmx.remote.default.class.loader or jmx.remote.default.class.loader.name should be specified.");
        }
        if (value == null && value2 == null) {
            return Thread.currentThread().getContextClassLoader();
        }
        if (value != null) {
            if ((value instanceof ClassLoader)) {
                return (ClassLoader)value;
            }
            throw new IllegalArgumentException("ClassLoader object is not an instance of " + ClassLoader.class.getName() + " : " + value.getClass().getName());
        }
        else if ((value2 instanceof ObjectName)) {
            ObjectName objectName = (ObjectName)value2;
            if (mBeanServer == null) {
                throw new IllegalArgumentException("Null MBeanServer object");
            }
            return mBeanServer.getClassLoader(objectName);
        }
        else {
            throw new IllegalArgumentException("ClassLoader name is not an instance of " + ObjectName.class.getName() + " : " + value2.getClass().getName());
        }
    }
    
    public static ClassLoader resolveClientClassLoader(Map<String, ?> map) {
        if (map == null) {
            return Thread.currentThread().getContextClassLoader();
        }
        Object value = map.get("jmx.remote.default.class.loader");
        if (value == null) {
            return Thread.currentThread().getContextClassLoader();
        }
        if ((value instanceof ClassLoader)) {
            return (ClassLoader)value;
        }
        throw new IllegalArgumentException("ClassLoader object is not an instance of " + ClassLoader.class.getName() + " : " + value.getClass().getName());
    }
    
    public static <T extends Throwable> T initCause(T t, Throwable cause) {
        t.initCause(cause);
        return t;
    }
    
    public static Throwable getCause(Throwable obj) {
        try {
            obj = (Throwable)obj.getClass().getMethod("getCause", (Class<?>[])null).invoke(obj, (Object[])null);
        }
        catch (Exception ex) {}
        return (obj != null) ? obj : obj;
    }
    
    public static int getNotifBufferSize(Map<String, ?> map) {
        int n = 1000;
        try {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.remote.x.notification.buffer.size"));
            if (s != null) {
                n = Integer.parseInt(s);
            }
            else {
                String s2 = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.remote.x.buffer.size"));
                if (s2 != null) {
                    n = Integer.parseInt(s2);
                }
            }
        }
        catch (RuntimeException obj) {
            EnvHelp.logger.warning("getNotifBufferSize", "Can't use System property jmx.remote.x.notification.buffer.size: " + obj);
            EnvHelp.logger.debug("getNotifBufferSize", obj);
        }
        try {
            if (map.containsKey("jmx.remote.x.notification.buffer.size")) {
                n = (int)getIntegerAttribute(map, "jmx.remote.x.notification.buffer.size", (long)n, 0L, 2147483647L);
            }
            else {
                n = (int)getIntegerAttribute(map, "jmx.remote.x.buffer.size", (long)n, 0L, 2147483647L);
            }
        }
        catch (RuntimeException obj2) {
            EnvHelp.logger.warning("getNotifBufferSize", "Can't determine queuesize (using default): " + obj2);
            EnvHelp.logger.debug("getNotifBufferSize", obj2);
        }
        return n;
    }
    
    public static int getMaxFetchNotifNumber(Map<String, ?> map) {
        return (int)getIntegerAttribute(map, "jmx.remote.x.notification.fetch.max", 1000L, 1L, 2147483647L);
    }
    
    public static long getFetchTimeout(Map<String, ?> map) {
        return getIntegerAttribute(map, "jmx.remote.x.notification.fetch.timeout", 60000L, 0L, Long.MAX_VALUE);
    }
    
    public static NotificationAccessController getNotificationAccessController(Map<String, ?> map) {
        return (map == null) ? null : ((NotificationAccessController)map.get("com.sun.jmx.remote.notification.access.controller"));
    }
    
    public static long getIntegerAttribute(Map<String, ?> map, String str, long n, long lng, long lng2) {
        Object value;
        if (map == null || (value = map.get(str)) == null) {
            return n;
        }
        long n2;
        if ((value instanceof Number)) {
            n2 = ((Number)value).longValue();
        }
        else if ((value instanceof String)) {
            n2 = Long.parseLong((String)value);
        }
        else {
            throw new IllegalArgumentException("Attribute " + str + " value must be Integer or String: " + value);
        }
        if (n2 < lng) {
            throw new IllegalArgumentException("Attribute " + str + " value must be at least " + lng + ": " + n2);
        }
        if (n2 > lng2) {
            throw new IllegalArgumentException("Attribute " + str + " value must be at most " + lng2 + ": " + n2);
        }
        return n2;
    }
    
    public static void checkAttributes(Map<?, ?> map) {
        for (Object next : map.keySet()) {
            if (!(next instanceof String)) {
                throw new IllegalArgumentException("Attributes contain key that is not a string: " + next);
            }
            continue;
        }
    }
    
    public static <V> Map<String, V> filterAttributes(Map<String, V> m) {
        if (EnvHelp.logger.traceOn()) {
            EnvHelp.logger.trace("filterAttributes", "starts");
        }
        TreeMap treeMap = new TreeMap(m);
        purgeUnserializable(treeMap.values());
        hideAttributes(treeMap);
        return treeMap;
    }
    
    private static void purgeUnserializable(Collection<?> collection) {
        EnvHelp.logger.trace("purgeUnserializable", "starts");
        Object o = null;
        int n = 0;
        for (Iterator iterator = collection.iterator(); iterator.hasNext(); ++n) {
            Object next = iterator.next();
            if (next == null || next instanceof String) {
                if (EnvHelp.logger.traceOn()) {
                    EnvHelp.logger.trace("purgeUnserializable", "Value trivially serializable: " + next);
                }
            }
            else {
                try {
                    if (o == null) {
                        o = new ObjectOutputStream(new EnvHelp.SinkOutputStream());
                    }
                    o.writeObject(next);
                    if (EnvHelp.logger.traceOn()) {
                        EnvHelp.logger.trace("purgeUnserializable", "Value serializable: " + next);
                    }
                }
                catch (IOException obj) {
                    if (EnvHelp.logger.traceOn()) {
                        EnvHelp.logger.trace("purgeUnserializable", "Value not serializable: " + next + ": " + obj);
                    }
                    iterator.remove();
                    o = null;
                }
            }
        }
    }
    
    private static void hideAttributes(SortedMap<String, ?> sortedMap) {
        if (sortedMap.isEmpty()) {
            return;
        }
        String str = (String)sortedMap.get("jmx.remote.x.hidden.attributes");
        Object defaultHiddenStrings;
        Object defaultHiddenPrefixes;
        if (str != null) {
            String s2;
            if (str.startsWith("=")) {
                s2 = str.substring(1);
            }
            else {
                s2 = str + " java.naming.security.* jmx.remote.authenticator jmx.remote.context jmx.remote.default.class.loader jmx.remote.message.connection.server jmx.remote.object.wrapping jmx.remote.rmi.client.socket.factory jmx.remote.rmi.server.socket.factory jmx.remote.sasl.callback.handler jmx.remote.tls.socket.factory jmx.remote.x.access.file jmx.remote.x.password.file ";
            }
            defaultHiddenStrings = new TreeSet();
            defaultHiddenPrefixes = new TreeSet();
            parseHiddenAttributes(s2, defaultHiddenStrings, defaultHiddenPrefixes);
        }
        else {
            String s = "java.naming.security.* jmx.remote.authenticator jmx.remote.context jmx.remote.default.class.loader jmx.remote.message.connection.server jmx.remote.object.wrapping jmx.remote.rmi.client.socket.factory jmx.remote.rmi.server.socket.factory jmx.remote.sasl.callback.handler jmx.remote.tls.socket.factory jmx.remote.x.access.file jmx.remote.x.password.file ";
            synchronized (EnvHelp.defaultHiddenStrings) {
                if (EnvHelp.defaultHiddenStrings.isEmpty()) {
                    parseHiddenAttributes(s, EnvHelp.defaultHiddenStrings, EnvHelp.defaultHiddenPrefixes);
                }
                defaultHiddenStrings = EnvHelp.defaultHiddenStrings;
                defaultHiddenPrefixes = EnvHelp.defaultHiddenPrefixes;
            }
        }
        String string = (String)sortedMap.lastKey() + "X";
        Iterator<Object> iterator = sortedMap.keySet().iterator();
        Iterator<Object> iterator2 = defaultHiddenStrings.iterator();
        Iterator<Object> iterator3 = defaultHiddenPrefixes.iterator();
        String s3;
        if (iterator2.hasNext()) {
            s3 = (String)iterator2.next();
        }
        else {
            s3 = string;
        }
        String prefix;
        if (iterator3.hasNext()) {
            prefix = (String)iterator3.next();
        }
        else {
            prefix = string;
        }
        while (iterator.hasNext()) {
            String s4 = (String)iterator.next();
            int compareTo;
            while ((compareTo = s3.compareTo(s4)) < 0) {
                if (iterator2.hasNext()) {
                    s3 = (String)iterator2.next();
                }
                else {
                    s3 = string;
                }
            }
            if (compareTo == 0) {
                iterator.remove();
            }
            else {
                while (prefix.compareTo(s4) <= 0) {
                    if (s4.startsWith(prefix)) {
                        iterator.remove();
                        break;
                    }
                    if (iterator3.hasNext()) {
                        prefix = (String)iterator3.next();
                    }
                    else {
                        prefix = string;
                    }
                }
            }
        }
        return;
    }
    
    private static void parseHiddenAttributes(String str, SortedSet<String> set, SortedSet<String> set2) {
        StringTokenizer stringTokenizer = new StringTokenizer(str);
        while (stringTokenizer.hasMoreTokens()) {
            String nextToken = stringTokenizer.nextToken();
            if (nextToken.endsWith("*")) {
                set2.add(nextToken.substring(0, nextToken.length() - 1));
            }
            else {
                set.add(nextToken);
            }
        }
    }
    
    public static long getServerConnectionTimeout(Map<String, ?> map) {
        return getIntegerAttribute(map, "jmx.remote.x.server.connection.timeout", 120000L, 0L, Long.MAX_VALUE);
    }
    
    public static long getConnectionCheckPeriod(Map<String, ?> map) {
        return getIntegerAttribute(map, "jmx.remote.x.client.connection.check.period", 60000L, 0L, Long.MAX_VALUE);
    }
    
    public static boolean computeBooleanFromString(String s) {
        return computeBooleanFromString(s, false);
    }
    
    public static boolean computeBooleanFromString(String str, boolean b) {
        if (str == null) {
            return b;
        }
        if (str.equalsIgnoreCase("true")) {
            return true;
        }
        if (str.equalsIgnoreCase("false")) {
            return false;
        }
        throw new IllegalArgumentException("Property value must be \"true\" or \"false\" instead of \"" + str + "\"");
    }
    
    public static <K, V> Hashtable<K, V> mapToHashtable(Map<K, V> m) {
        HashMap t = new HashMap(m);
        if (t.containsKey(null)) {
            t.remove(null);
        }
        Iterator<Object> iterator = t.values().iterator();
        while (iterator.hasNext()) {
            if (iterator.next() != null) {
                continue;
            }
            iterator.remove();
        }
        return new Hashtable(t);
    }
    
    public static boolean isServerDaemon(Map<String, ?> map) {
        return map != null && "true".equalsIgnoreCase((String)map.get("jmx.remote.x.daemon"));
    }
    
    static {
        defaultHiddenStrings = new TreeSet();
        defaultHiddenPrefixes = new TreeSet();
        logger = new ClassLogger("javax.management.remote.misc", "EnvHelp");
    }
    
    private static final class SinkOutputStream extends OutputStream
    {
        @Override
        public void write(byte[] array, int n, int n2) {
        }
        
        @Override
        public void write(int n) {
        }
    }
}
